/**
  *  \file server/cfgout.cc
  *  \brief Server: Config Output
  *
  *  This file contains code to produce game configuration output in
  *  JSON format.
  */

#include "server/cfgout.h"
#include "cpluslib/format.h"
#include "server/jsonout.h"
#include "game/config.h"
#include "cpluslib/string.h"

void
writeConfig(JsonWriter& w, ConfigOption& opt)
{
    if (ConfigIntOption* io = dynamic_cast<ConfigIntOption*>(&opt)) {
        w.writeInt(strUCase(io->getName()), (*io)());
    } else if (ConfigIntArrayBaseOption* abo = dynamic_cast<ConfigIntArrayBaseOption*>(&opt)) {
        w.startArray(strUCase(abo->getName()));
        for (int i = 0, e = abo->getLimit(); i < e; ++i)
            w.writeInt(abo->getArray()[i]);
        w.endArray();
    } else if (ConfigStringOption* so = dynamic_cast<ConfigStringOption*>(&opt)) {
        w.writeString(strUCase(so->getName()), (*so)());
    } else if (ConfigCostOption* co = dynamic_cast<ConfigCostOption*>(&opt)) {
        w.startArray(strUCase(co->getName()));
        for (int i = 1; i <= NUM_PLAYERS; ++i) {
            w.startObject();
            const GCost& pc = (*co)(i);
            w.writeInt("T", pc.get(el_Tritanium));
            w.writeInt("D", pc.get(el_Duranium));
            w.writeInt("M", pc.get(el_Molybdenum));
            w.writeInt("MC", pc.get(el_Money));
            w.writeInt("SUPPLIES", pc.get(el_Supplies));
            w.endObject();
        }
        w.endArray();
    } else {
        // what?
    }
}


/** Constructor.
    \param topic Selected config slice */
ServerConfigWriter::ServerConfigWriter(int topic)
    : topic(topic)
{ }

void
ServerConfigWriter::write(JsonWriter& w)
{
    w.startObject(format("cfg%d", topic));
    switch (topic) {
     case 0:
        for (Config::iterator i = config.begin(); i != config.end(); ++i)
            writeConfig(w, *i);
        break;
     case 1:
        // Options that potentially affect building and taxation on planets
        writeConfig(w, config.StarbaseCost);
        writeConfig(w, config.AllowScienceMissions);
        writeConfig(w, config.ClimateLimitsPopulation);
        writeConfig(w, config.CrystalsPreferDeserts);
        writeConfig(w, config.DefenseForUndetectable);
        writeConfig(w, config.DefenseToBlockBioscan);
        writeConfig(w, config.FactoriesForDetectable);
        writeConfig(w, config.MaxColTempSlope);
        writeConfig(w, config.MaxShipsHissing);
        writeConfig(w, config.MinesForDetectable);
        writeConfig(w, config.NativeClimateDeathRate);
        writeConfig(w, config.StructureDecayOnUnowned);
        writeConfig(w, config.TransuraniumDecayRate);
        writeConfig(w, config.AllowEatingSupplies);
        writeConfig(w, config.ClimateDeathRate);
        writeConfig(w, config.ColonistTaxRate);
        writeConfig(w, config.FighterSweepRange);
        writeConfig(w, config.GroundDefenseFactor);
        writeConfig(w, config.GroundKillFactor);
        writeConfig(w, config.HissEffectRate);
        writeConfig(w, config.MaxPlanetaryIncome);
        writeConfig(w, config.MaximumDefenseOnBase);
        writeConfig(w, config.NativeTaxRate);
        writeConfig(w, config.ProductionRate);
        writeConfig(w, config.RaceGrowthRate);
        writeConfig(w, config.RaceMiningRate);
        writeConfig(w, config.StructureDecayPerTurn);
        writeConfig(w, config.TerraformRate);
        break;
    }
    w.endObject();
}

bool
ServerConfigWriter::isSame(ServerObjectWriter* other)
{
    ServerConfigWriter* p = dynamic_cast<ServerConfigWriter*>(other);
    return p != 0 && p->topic == topic;
}
